@@ -1,6 +1,9 @@
 enum GlobalEnum { globalenum1=1, globalenum2 };
 
-GlobalEnum global_test(GlobalEnum e) { return e; }
+/* This function should be static as it will be emitted into the code for
+ * every module.  All _static targets would fail here with a multiple 
+ * definition if this is not static. */
+static GlobalEnum global_test(GlobalEnum e) { return e; }
 
 class A { 
  public: 
