@@ -11,7 +11,7 @@
  *
  * See the file LICENSE for information on usage and redistribution.
  *
- * $Header: /cvsroot/SWIG/Source/Modules/swigmod.h,v 1.19 2003/12/28 21:47:58 cheetah Exp $
+ * $Header: /cvsroot/swig/SWIG/Source/Modules/swigmod.h,v 1.34 2004/12/06 22:53:09 wsfulton Exp $
  * ----------------------------------------------------------------------------- */
 
 #ifndef SWIGMOD_H_
@@ -25,6 +25,12 @@
 #include "preprocessor.h"
 #include "swigwarn.h"
 
+#if !defined(HAVE_BOOL)
+typedef int bool;
+#define true ((bool)1)
+#define false ((bool)0)
+#endif
+
 #define NOT_VIRTUAL     0
 #define PLAIN_VIRTUAL   1
 #define PURE_VIRTUAL    2
@@ -34,13 +40,13 @@ extern  int       line_number;
 extern  int       start_line;
 extern  int       CPlusPlus;                        // C++ mode
 extern  int       Extend;                           // Extend mode
-extern  int       NoInclude;                        // NoInclude flag
 extern  int       Verbose;
 extern  int       IsVirtual;
 extern  int       ImportMode;
 extern  int       NoExcept;                         // -no_except option
 extern  int       Abstract;                         // abstract base class
 extern  int       SmartPointer;                     // smart pointer methods being emitted
+extern  int       SwigRuntime;
 
 /* Overload "argc" and "argv" */
 extern String *argv_template_string;
@@ -54,7 +60,8 @@ extern String *argc_template_string;
 
 class Dispatcher {
  public:
-  
+
+  Dispatcher() : cplus_mode(PUBLIC) {}
   virtual int emit_one(Node *n);
   virtual int emit_children(Node *n);
   virtual int defaultHandler(Node *n);
@@ -86,6 +93,7 @@ class Dispatcher {
   virtual int externDeclaration(Node *n);
   virtual int enumDeclaration(Node *n);
   virtual int enumvalueDeclaration(Node *n);
+  virtual int enumforwardDeclaration(Node *n);
   virtual int classDeclaration(Node *n);
   virtual int classforwardDeclaration(Node *n);
   virtual int constructorDeclaration(Node *n);
@@ -94,6 +102,11 @@ class Dispatcher {
   virtual int usingDeclaration(Node *n);
   virtual int namespaceDeclaration(Node *n);
   virtual int templateDeclaration(Node *n);
+
+  enum AccessMode { PUBLIC, PRIVATE, PROTECTED };
+
+protected:
+  AccessMode cplus_mode;
 };
 
 /************************************************************************
@@ -142,6 +155,7 @@ public:
   virtual int externDeclaration(Node *n);
   virtual int enumDeclaration(Node *n);
   virtual int enumvalueDeclaration(Node *n);
+  virtual int enumforwardDeclaration(Node *n);
   virtual int classDeclaration(Node *n);
   virtual int classforwardDeclaration(Node *n);
   virtual int constructorDeclaration(Node *n);
@@ -192,21 +206,24 @@ public:
                                    Node *parent, 
                                    Hash *vm, 
                                    int default_director, 
-                                   int &virtual_destructor);
+                                   int &virtual_destructor,
+				   int protectedbase = 0);
   virtual int classDirectorConstructor(Node *n);
   virtual int classDirectorDefaultConstructor(Node *n);
   virtual int classDirectorMethod(Node *n, Node *parent, String *super);
   virtual int classDirectorConstructors(Node *n);
+  virtual int classDirectorDestructor(Node *n);
   virtual int classDirectorMethods(Node *n);
   virtual int classDirectorDisown(Node *n);
 
   /* Miscellaneous */
-
-  virtual  int  validIdentifier(String *s);        /* valid identifier? */
-  virtual  int  addSymbol(String *s, Node *n);     /* Add symbol        */
-  virtual  Node *symbolLookup(String *s);          /* Symbol lookup     */
-  virtual  Node *classLookup(SwigType *s);         /* Class lookup      */
-  virtual  int  abstractClassTest(Node *n);	   /* Is class really abstract? */
+  virtual  int  validIdentifier(String *s);                 /* valid identifier? */
+  virtual  int  addSymbol(const String *s, const Node *n);  /* Add symbol        */
+  virtual  Node *symbolLookup(String *s);                   /* Symbol lookup     */
+  virtual  Node *classLookup(SwigType *s);                  /* Class lookup      */
+  virtual  Node *enumLookup(SwigType *s);                   /* Enum lookup       */
+  virtual  int  abstractClassTest(Node *n);	            /* Is class really abstract? */
+  virtual  int  is_assignable(Node *n);	                   /* Is variable assignable? */
   
   /* Allow director related code generation */
   void allow_directors(int val = 1);
@@ -220,16 +237,19 @@ public:
   /* Returns the dirprot mode */
   int dirprot_mode() const;  
 
+  /* Check if the non public constructor is  needed (for directors) */
+  int need_nonpublic_ctor(Node *n);
+
+  /* Check if the non public member is  needed (for directors) */
+  int need_nonpublic_member(Node *n);
+
   /* Set none comparison string */
   void setSubclassInstanceCheck(String *s);
 
   /* Set overload variable templates argc and argv */
   void setOverloadResolutionTemplates(String *argc, String *argv);
 
- protected:
-  /* Patch C++ pass-by-value */
-  static void patch_parms(Parm *p);
-
+protected:
   /* Allow multiple-input typemaps */
   void   allow_multiple_input(int val = 1);
 
@@ -241,10 +261,13 @@ public:
 
   /* Return the node for the current class */
   Node *getCurrentClass() const;
-    
+
+  /* Return C++ mode */
+  int getCPlusMode() const;
+
   /* Return the real name of the current class */
   String *getClassName() const;
-  
+
   /* Return the current class prefix */
   String *getClassPrefix() const;
 
@@ -260,13 +283,22 @@ public:
   /* Director constructor "template" code */
   String *director_ctor_code;
 
- private:
+  /* Director 'protected' constructor "template" code*/
+  String *director_prot_ctor_code;
+
+  /* Director allows multiple inheritance */
+  int director_multiple_inheritance;
+
+  /* Director language module */
+  int director_language;
+
+private:
   Hash   *symbols;
   Hash   *classtypes;
+  Hash   *enumtypes;
   int     overloading;
   int     multiinput;
   int     directors;
-  
 };
 
 int   SWIG_main(int, char **, Language *);
@@ -283,7 +315,7 @@ int    emit_isvarargs(ParmList *);
 void   emit_attach_parmmaps(ParmList *, Wrapper *f);
 void   emit_mark_varargs(ParmList *l);
 void   emit_action(Node *n, Wrapper *f);
-List  *Swig_overload_rank(Node *n);
+void   Swig_overload_check(Node *n);
 String *Swig_overload_dispatch(Node *n, const String_or_char *fmt, int *);
 SwigType *cplus_value_type(SwigType *t);
 
@@ -312,8 +344,11 @@ int is_protected(Node* n);
 int is_member_director(Node* parentnode, Node* member); 
 int is_member_director(Node* member); 
 void Wrapper_virtual_elimination_mode_set(int);
+void Wrapper_director_mode_set(int);
 void Wrapper_director_protected_mode_set(int);
+void Wrapper_template_extmode_set(int);
 
+void clean_overloaded(Node *n);
 
 /* Contracts */
 
