@@ -21,7 +21,7 @@
  * See the file LICENSE for information on usage and redistribution.	
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_typepass_cxx[] = "$Header: /cvsroot/SWIG/Source/Modules/typepass.cxx,v 1.8 2004/01/09 19:36:58 beazley Exp $";
+char cvsroot_typepass_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/typepass.cxx,v 1.21 2004/12/14 21:25:12 wsfulton Exp $";
 
 #include "swigmod.h"
 
@@ -34,7 +34,8 @@ struct normal_node {
 
 static normal_node *patch_list = 0;
 
-class TypePass : public Dispatcher {
+/* Singleton class - all non-static methods in this class are private */
+class TypePass : private Dispatcher {
   Node   *inclass;
   Node   *module;
   int    importmode;
@@ -42,18 +43,13 @@ class TypePass : public Dispatcher {
   Hash   *classhash;
   List   *normalize;
 
+  TypePass() {
+  }
+
   /* Normalize a type. Replaces type with fully qualified version */
 
   void normalize_type(SwigType *ty) {
     SwigType *qty;
-    /*qty = Swig_symbol_type_qualify(ty,0);*/
-    /*      if (SwigType_istemplate(ty)) {
-	    qty = Swig_symbol_type_qualify(ty,0);
-	    Clear(ty);
-	    Append(ty,qty);
-	    }
-    */
-
     if (CPlusPlus) {
 	Replaceall(ty,"struct ","");
 	Replaceall(ty,"union ","");
@@ -131,16 +127,18 @@ class TypePass : public Dispatcher {
     }
     Swig_symbol_setscope(currentsym);
   }
-  
+
   /* generate C++ inheritance type-relationships */
-  void cplus_inherit_types(Node *first, Node *cls, String *clsname, String *cast = 0) {
+  void cplus_inherit_types_impl(Node *first, Node *cls, String*clsname,
+				const char *bases, const char *baselist,
+				int ispublic, String *cast = 0) {
       
     if (first == cls) return;  /* The Marcelo check */
     if (!cls) cls = first;
 
-    List *ilist = Getattr(cls,"bases");
+    List *ilist = Getattr(cls,bases);
     if (!ilist) {
-      List *nlist = Getattr(cls,"baselist");
+      List *nlist = Getattr(cls,baselist);
       if (nlist) {
 	int     len = Len(nlist);
 	int i;
@@ -180,9 +178,11 @@ class TypePass : public Dispatcher {
 		  } 
 		}
 		if (Strcmp(nodeType(bcls),"classforward") != 0) {
-		  Swig_error(Getfile(bname),Getline(bname),"'%s' is not a class. \n",bname);
+		  Swig_error(Getfile(cls),Getline(cls),"'%s' does not have a valid base class.\n", Getattr(cls,"name"));
+		  Swig_error(Getfile(bcls),Getline(bcls),"'%s' is not a valid base class.\n", bname);
 		} else {
-		  Swig_warning(WARN_TYPE_INCOMPLETE,Getfile(bname),Getline(bname),"Base class '%s' is incomplete.\n", bname);
+		  Swig_warning(WARN_TYPE_INCOMPLETE,Getfile(cls),Getline(cls),"Base class '%s' is incomplete.\n", bname);
+		  Swig_warning(WARN_TYPE_INCOMPLETE,Getfile(bcls),Getline(bcls),"Only forward declaration '%s' was found.\n", bname);
 		  clsforward = 1;
 		}
 		bcls = 0;
@@ -191,7 +191,8 @@ class TypePass : public Dispatcher {
 		  if (!ilist) ilist = NewList();
 		  Append(ilist,bcls);
 		} else {
-		  Swig_error(Getfile(bcls),Getline(bcls),"class '%s' must be defined before it is used as a base class.\n", bname);
+		  Swig_warning(WARN_TYPE_UNDEFINED_CLASS,Getfile(cls),Getline(cls),"Base class '%s' undefined.\n", bname);
+		  Swig_warning(WARN_TYPE_UNDEFINED_CLASS,Getfile(bcls),Getline(bcls),"'%s' must be defined before it is used as a base class.\n", bname);
 		}
 	      }
 	    }
@@ -201,10 +202,10 @@ class TypePass : public Dispatcher {
 	  if (tname) Delete(tname);
 	  if (!bcls) {
 	    if (!clsforward) {
-	      if (!Getmeta(bname,"already_warned")) {
-		Swig_warning(WARN_TYPE_UNDEFINED_CLASS, Getfile(bname),Getline(bname),"Nothing known about class '%s'. Ignored.\n", SwigType_namestr(bname));
+	      if (ispublic && !Getmeta(bname,"already_warned")) {
+		Swig_warning(WARN_TYPE_UNDEFINED_CLASS, Getfile(cls),Getline(cls),"Nothing known about base class '%s'. Ignored.\n", SwigType_namestr(bname));
 		if (Strchr(bname,'<')) {
-		  Swig_warning(WARN_TYPE_UNDEFINED_CLASS, Getfile(bname), Getline(bname), "Maybe you forgot to instantiate '%s' using %%template.\n", SwigType_namestr(bname));
+		  Swig_warning(WARN_TYPE_UNDEFINED_CLASS, Getfile(cls), Getline(cls), "Maybe you forgot to instantiate '%s' using %%template.\n", SwigType_namestr(bname));
 		}
 		Setmeta(bname,"already_warned","1");
 	      }
@@ -214,7 +215,7 @@ class TypePass : public Dispatcher {
 	}
       }
       if (ilist) {
-	Setattr(cls,"bases",ilist);
+	Setattr(cls,bases,ilist);
       }
     }
     if (!ilist) return;
@@ -244,81 +245,36 @@ class TypePass : public Dispatcher {
 
       /* Recursively hit base classes */
       String *newcast = NewStringf("(%s *)%s", SwigType_namestr(Getattr(bclass,"name")), cast);
-      cplus_inherit_types(first,bclass,clsname, newcast);
+      cplus_inherit_types_impl(first,bclass,clsname,bases,baselist,ispublic,newcast);
       Delete(newcast);
     }
   }
 
-  /* Clean overloaded list.  Removes templates, friends, ignored, and errors */
-
-  void clean_overloaded(Node *n) {
-    Node *nn = Getattr(n,"sym:overloaded");
-    Node *first = 0;
-    int   cnt = 0;
-    while (nn) {
-      if ((Strcmp(nodeType(nn),"template") == 0) ||
-	  (Getattr(nn,"feature:ignore")) ||
-	  (Getattr(nn,"error")) ||
-	  ((Strcmp(nodeType(nn),"using") == 0) && !firstChild(nn)) ||
-	  (checkAttribute(nn,"storage","friend"))) {
-	/* Remove from overloaded list */
-	Node *ps = Getattr(nn,"sym:previousSibling");
-	Node *ns = Getattr(nn,"sym:nextSibling");
-	if (ps) {
-	  Setattr(ps,"sym:nextSibling",ns);
-	} 
-	if (ns) {
-	  Setattr(ns,"sym:previousSibling",ps);
-	}
-	Delattr(nn,"sym:previousSibling");
-	Delattr(nn,"sym:nextSibling");
-	Delattr(nn,"sym:overloaded");
-	nn = ns;
-	continue;
-      } else if ((Strcmp(nodeType(nn),"using") == 0)) {
-	/* A possibly dangerous parse tree hack.  We're going to
-           cut the parse tree node out and stick in the resolved
-           using declarations */
-	
-	Node *ps = Getattr(nn,"sym:previousSibling");
-	Node *ns = Getattr(nn,"sym:nextSibling");
-	Node *un = firstChild(nn);
-	Node *pn = un;
-
-	if (!first) {
-	  first = un;
-	}
-	while (pn) {
-	  Node *ppn = Getattr(pn,"sym:nextSibling");
-	  Setattr(pn,"sym:overloaded",first);
-	  Setattr(pn,"sym:overname", NewStringf("%s_%d", Getattr(nn,"sym:overname"), cnt++));
-	  if (ppn) pn = ppn;
-	  else break;
-	}
-	if (ps) {
-	  Setattr(ps,"sym:nextSibling",un);
-	  Setattr(un,"sym:previousSibling",ps);
-	}
-	if (ns) {
-	  Setattr(ns,"sym:previousSibling", pn);
-	  Setattr(pn,"sym:nextSibling",ns);
-	}
-	if (!first) {
-	  first = un;
-	  Setattr(nn,"sym:overloaded",first);
-	}
-      } else {
-	if (!first) first = nn;
-	Setattr(nn,"sym:overloaded",first);
+  void append_list(List *lb, List *la) {
+    if (la && lb) {
+      for (Iterator bi = First(la); bi.item; bi = Next(bi)) {
+	Append(lb,bi.item);
       }
-      nn = Getattr(nn,"sym:nextSibling");
-    }
-    if (!first || (first && !Getattr(first,"sym:nextSibling"))) {
-      Delattr(n,"sym:overloaded");
     }
   }
 
-public:
+  void cplus_inherit_types(Node *first, Node *cls, String *clsname, String *cast = 0) {
+    cplus_inherit_types_impl(first, cls, clsname, "bases", "baselist", 1, cast);
+    cplus_inherit_types_impl(first, cls, clsname, "protectedbases","protectedbaselist", 0, cast);
+    cplus_inherit_types_impl(first, cls, clsname, "privatebases"  ,"privatebaselist", 0, cast);
+
+    if (!cls) cls = first;
+
+    List *allbases = NewList();
+    append_list(allbases,Getattr(cls,"bases"));
+    append_list(allbases,Getattr(cls,"protectedbases"));
+    append_list(allbases,Getattr(cls,"privatebases"));
+    if (Len(allbases)) {
+      Setattr(cls,"allbases",allbases);
+    } else {
+      Delete(allbases);
+    }
+  }
 
   /* ------------------------------------------------------------
    * top()
@@ -400,13 +356,19 @@ public:
 	fname = SwigType_typedef_resolve_all(name);
 	if (Strcmp(fname,name) != 0) {
 	  cn = Swig_symbol_clookup_local(fname,0);
-	  if ((!cn) || (Strcmp(nodeType(cn),"template") == 0)) {
+	  if ((!cn) || (Strcmp(nodeType(cn),"template") == 0)
+	      || (Getattr(cn,"feature:onlychildren") != 0)
+	      || (Getattr(n,"feature:onlychildren") != 0)) {
 	    Swig_symbol_cadd(fname,n);
 	    SwigType_typedef_class(fname);
 	    scopename = Copy(fname);
 	  } else {
-	    Swig_warning(WARN_TYPE_REDEFINED,Getfile(n),Getline(n),"Template '%s' was already wrapped as '%s' at %s:%d.\n",
-			 SwigType_namestr(name), SwigType_namestr(Getattr(cn,"name")), Getfile(cn), Getline(cn));
+	    Swig_warning(WARN_TYPE_REDEFINED,Getfile(n),Getline(n),
+			 "Template '%s' was already wrapped,\n",
+			 SwigType_namestr(name));
+	    Swig_warning(WARN_TYPE_REDEFINED,Getfile(cn), Getline(cn),
+			 "previous wrap of '%s'.\n",
+			 SwigType_namestr(Getattr(cn,"name")));
 	    scopename = 0;
 	  }
 	} else {
@@ -737,20 +699,56 @@ public:
 
   virtual int enumDeclaration(Node *n) {
     String *name = Getattr(n,"name");
-    String *uname = Getattr(n,"unnamed");
+
     if (name) {
-      SwigType *t;
-      if (uname) {
-	t = NewStringf("enum %s", uname);
-	if (checkAttribute(n,"storage","typedef")) {
-	  SwigType_typedef(t,name);
-	}
+      String *scope = 0;
+
+      // Add a typedef to the type table so that we can use 'enum Name' as well as just 'Name'
+      if (nsname || inclass) {
+
+        // But first correct the name and tdname to contain the fully qualified scopename
+        if (nsname && inclass) {
+          scope = NewStringf("%s::%s", nsname, Getattr(inclass,"name"));
+        } else if (nsname) {
+          scope = NewStringf("%s", nsname);
+        } else if (inclass) {
+          scope = NewStringf("%s", Getattr(inclass,"name"));
+        }
+
+        String *nname = NewStringf("%s::%s", scope, name);
+        Setattr(n,"name",nname);
+
+        String *tdname = Getattr(n,"tdname");
+        if (tdname) {
+          tdname = NewStringf("%s::%s", scope, tdname);
+          Setattr(n,"tdname",tdname);
+        }
+
+        SwigType *t = NewStringf("enum %s", nname);
+        SwigType_typedef(t,name);
       } else {
-	t = NewStringf("enum %s", name);
-	SwigType_typedef(t,name);
+        SwigType *t = NewStringf("enum %s", name);
+        SwigType_typedef(t,name);
       }
-      Delete(t);
+      Delete(scope);
     }
+
+    String *tdname = Getattr(n,"tdname");
+    String *unnamed = Getattr(n,"unnamed");
+    String *storage = Getattr(n,"storage");
+
+    // Construct enumtype - for declaring an enum of this type with SwigType_ltype() etc
+    String *enumtype = 0;
+    if (unnamed && tdname && (Cmp(storage,"typedef") == 0)) {
+      enumtype = Copy(Getattr(n,"tdname"));
+    } else if (name) {
+      enumtype = NewStringf("%s%s", CPlusPlus ? "" : "enum ", Getattr(n,"name"));
+    } else {
+      // anonymous enums
+      enumtype = Copy(Getattr(n,"type"));
+    }
+    Setattr(n,"enumtype",enumtype);
+
     emit_children(n);
     return SWIG_OK;
   }
@@ -773,9 +771,30 @@ public:
       Setattr(n,"value",new_value);
       Delete(new_value);
     }
+
+    // Make up an enumvalue if one was not specified in the parsed code
+    if (Getattr(n,"_last") && !Getattr(n,"enumvalue")) { // Only the first enum item has _last set
+      Setattr(n,"enumvalueex", "0");
+    }
+    Node *next = nextSibling(n);
+    if (next && !Getattr(next,"enumvalue")) {
+      Setattr(next,"enumvalueex", NewStringf("%s + 1", Getattr(n,"sym:name")));
+    }
+
     return SWIG_OK;
   }
 
+  /* ------------------------------------------------------------
+   * enumforwardDeclaration()
+   * ------------------------------------------------------------ */ 
+
+  virtual int enumforwardDeclaration(Node *n) {
+
+    // Use enumDeclaration() to do all the hard work.
+    // Note that no children can be emitted in a forward declaration as there aren't any.
+    return enumDeclaration(n);
+  }
+
   /* ------------------------------------------------------------
    * usingDeclaration()
    * ------------------------------------------------------------ */
@@ -971,13 +990,17 @@ public:
     }
     return SWIG_OK;
   }
+
+public:
+  static void pass(Node *n) {
+    TypePass t;
+    t.top(n);
+  }
 };
 
 void Swig_process_types(Node *n) {
   if (!n) return;
-  TypePass *t = new TypePass;
-  t->top(n);
-  delete t;
+  TypePass::pass(n);
 }
 
 
