@@ -14,7 +14,7 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_parms_c[] = "$Header: /cvsroot/SWIG/Source/Swig/parms.c,v 1.7 2003/09/04 23:26:29 cheetah Exp $";
+char cvsroot_parms_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/parms.c,v 1.11 2004/11/04 22:10:19 wsfulton Exp $";
 
 #include "swig.h"
 
@@ -26,7 +26,7 @@ char cvsroot_parms_c[] = "$Header: /cvsroot/SWIG/Source/Swig/parms.c,v 1.7 2003/
  * Create a new parameter from datatype 'type' and name 'n'.
  * ------------------------------------------------------------------------ */
 
-Parm *NewParm(SwigType *type, String_or_char *n) {
+Parm *NewParm(SwigType *type, const String_or_char *n) {
   Parm *p = NewHash();
   
   if (type) {
@@ -48,6 +48,7 @@ Parm *CopyParm(Parm *p) {
   String   *ignore;
   String   *alttype;
   String   *byname;
+  String   *compactdefargs;
 
   Parm *np = NewHash();
   t = Getattr(p,"type");
@@ -57,6 +58,7 @@ Parm *CopyParm(Parm *p) {
   ignore = Getattr(p,"ignore");
   alttype = Getattr(p,"alttype");
   byname = Getattr(p, "arg:byname");
+  compactdefargs = Getattr(p, "compactdefargs");
 
   if (t) 
     Setattr(np,"type",Copy(t));
@@ -72,6 +74,8 @@ Parm *CopyParm(Parm *p) {
     Setattr(np,"alttype", Copy(alttype));
   if (byname)
     Setattr(np, "arg:byname", Copy(byname));
+  if (compactdefargs)
+    Setattr(np, "compactdefargs", Copy(compactdefargs));
       
   Setfile(np,Getfile(p));
   Setline(np,Getline(p));
@@ -169,7 +173,32 @@ String *ParmList_str(ParmList *p) {
 }
 
 /* ---------------------------------------------------------------------
- * ParmList_str()
+ * ParmList_str_defaultargs()
+ *
+ * Generates a string of parameters including default arguments
+ * ---------------------------------------------------------------------- */
+
+String *ParmList_str_defaultargs(ParmList *p) {
+  String *out;
+  String *value;
+  SwigType *t;
+
+  out = NewString("");
+  while(p) {
+    t = Getattr(p,"type");
+    Printf(out,"%s", SwigType_str(t,Getattr(p,"name")));
+    value = Getattr(p,"value");
+    if (value)
+      Printf(out,"=%s", value);
+    p = nextSibling(p);
+    if (p)
+      Printf(out,",");
+  }
+  return out;
+}
+
+/* ---------------------------------------------------------------------
+ * ParmList_protostr()
  *
  * Generate a prototype string.
  * ---------------------------------------------------------------------- */
@@ -193,5 +222,74 @@ String *ParmList_protostr(ParmList *p) {
   return out;
 }
 
+/* ---------------------------------------------------------------------
+ * ParmList_is_compactdefargs()
+ *
+ * Returns 1 if the parameter list passed in is marked for compact argument
+ * handling (by the "compactdefargs" attribute). Otherwise returns 0.
+ * ---------------------------------------------------------------------- */
+
+int ParmList_is_compactdefargs(ParmList *p) {
+  int compactdefargs = 0;
+  
+  if (p) {
+    compactdefargs = Getattr(p,"compactdefargs") ? 1 : 0;
+
+    /* The "compactdefargs" attribute should only be set on the first parameter in the list.
+     * However, sometimes an extra parameter is inserted at the beginning of the parameter list,
+     * so we check the 2nd parameter too. */
+    if (!compactdefargs) {
+      Parm *nextparm = nextSibling(p);
+      compactdefargs = (nextparm && Getattr(nextparm,"compactdefargs")) ? 1 : 0;
+    }
+  }
+
+  return compactdefargs;
+}
 
 
+/* ---------------------------------------------------------------------
+ * ParmList_has_defaultargs()
+ *
+ * Returns 1 if the parameter list passed in is has one or more default
+ * arguments. Otherwise returns 0.
+ * ---------------------------------------------------------------------- */
+
+int ParmList_has_defaultargs(ParmList *p) {
+    int default_args = 0;
+    while (p) {
+      if (Getattr(p, "value")) {
+        default_args = 1;
+        break;
+      }
+      p = nextSibling(p);
+    }
+    return default_args;
+}
+
+/* ---------------------------------------------------------------------
+ * ParmList_copy_all_except_last_parm()
+ *
+ * Create a new parameter list by copying all the parameters barring the
+ * last parameter.
+ * ---------------------------------------------------------------------- */
+
+ParmList *ParmList_copy_all_except_last_parm(ParmList *p) {
+  ParmList* newparms = 0;
+  Parm *newparm = 0;
+  Parm *pp = 0;
+  Parm *fp = 0;
+  while (nextSibling(p)) {
+    newparm = CopyParm(p);
+    if (pp) {
+      set_nextSibling(pp,newparm);
+    } else {
+      fp = newparm;
+    }
+    pp = newparm;
+    p = nextSibling(p);
+  }
+  newparms = fp;
+  return newparms;
+}
+
