@@ -9,7 +9,7 @@
  * Copyright (C) 1999-2000.  The University of Chicago
  * See the file LICENSE for information on usage and redistribution.	
  *
- * $Header: /cvsroot/SWIG/Source/Swig/swig.h,v 1.78 2004/01/10 00:04:17 cheetah Exp $
+ * $Header: /cvsroot/swig/SWIG/Source/Swig/swig.h,v 1.98 2004/12/04 08:33:02 marcelomatus Exp $
  * ----------------------------------------------------------------------------- */
 
 #ifndef SWIGCORE_H_
@@ -24,6 +24,10 @@
 extern "C" {
 #endif
 
+#ifndef MACSWIG
+#include "swigconfig.h"
+#endif
+
 #include "doh.h"
 
 /* Status codes */
@@ -68,7 +72,12 @@ typedef  DOH     SwigType;
 #define   T_ULONGLONG  12
 #define   T_FLOAT      20
 #define   T_DOUBLE     21
-#define   T_NUMERIC    22
+#define   T_LONGDOUBLE 22
+#define   T_FLTCPLX    23
+#define   T_DBLCPLX    24
+#define   T_NUMERIC    25
+
+#define   T_COMPLEX    T_DBLCPLX
 
 /* non-numeric */
 
@@ -95,9 +104,12 @@ extern List    *Swig_search_path();
 extern FILE    *Swig_open(const String_or_char *name);
 extern String  *Swig_read_file(FILE *f);
 extern String  *Swig_include(const String_or_char *name);
+extern String  *Swig_include_sys(const String_or_char *name);
 extern int      Swig_insert_file(const String_or_char *name, File *outfile);
 extern void     Swig_set_config_file(const String_or_char *filename);
 extern String  *Swig_get_config_file(void);
+extern void     Swig_set_push_dir(int dopush);
+extern int      Swig_get_push_dir(void);
 extern void     Swig_swiglib_set(const String_or_char *);
 extern String  *Swig_swiglib_get();
 extern void     Swig_register_filebyname(const String_or_char *filename, File *outfile);
@@ -200,13 +212,15 @@ extern void         SwigScanner_idstart(SwigScanner *, char *idchar);
 extern SwigType   *NewSwigType(int typecode);
 extern void        SwigType_del_element(SwigType *t);
 extern void        SwigType_add_pointer(SwigType *t);
-extern void        SwigType_add_memberpointer(SwigType *t, String_or_char *qual);
+extern void        SwigType_add_memberpointer(SwigType *t, const String_or_char *qual);
 extern void        SwigType_del_pointer(SwigType *t);
-extern void        SwigType_add_array(SwigType *t, String_or_char *size);
+extern void        SwigType_add_array(SwigType *t, const String_or_char *size);
+extern void        SwigType_add_array(SwigType *t, const String_or_char *size);
+extern void        SwigType_del_array(SwigType *t);
 extern SwigType   *SwigType_pop_arrays(SwigType *t);
 extern void        SwigType_add_reference(SwigType *t);
 extern void        SwigType_del_reference(SwigType *t);
-extern void        SwigType_add_qualifier(SwigType *t, String_or_char *qual);
+extern void        SwigType_add_qualifier(SwigType *t, const String_or_char *qual);
 extern void        SwigType_del_qualifier(SwigType *t);
 extern void        SwigType_add_function(SwigType *t, ParmList *parms);
 extern void        SwigType_add_template(SwigType *t, ParmList *parms);
@@ -247,10 +261,11 @@ extern String     *SwigType_templateargs(SwigType *t);
 extern String     *SwigType_prefix(SwigType *t);
 extern int         SwigType_array_ndim(SwigType *t);
 extern String     *SwigType_array_getdim(SwigType *t, int n);
-extern void        SwigType_array_setdim(SwigType *t, int n, String_or_char *rep);
+extern void        SwigType_array_setdim(SwigType *t, int n, const String_or_char *rep);
 extern SwigType   *SwigType_array_type(SwigType *t);
 extern String     *SwigType_default(SwigType *t);
 extern void        SwigType_typename_replace(SwigType *t, String *pat, String *rep);
+extern SwigType   *SwigType_alttype(SwigType *t, int ltmap);
 
 /* --- Type-system managment --- */
 extern void        SwigType_typesystem_init();
@@ -261,7 +276,7 @@ extern void        SwigType_inherit(String *subclass, String *baseclass, String
 extern int         SwigType_issubtype(SwigType *subtype, SwigType *basetype);
 extern void        SwigType_scope_alias(String *aliasname, Typetab *t);
 extern void        SwigType_using_scope(Typetab *t);
-extern void        SwigType_new_scope(String_or_char *name);
+extern void        SwigType_new_scope(const String_or_char *name);
 extern void        SwigType_inherit_scope(Typetab *scope);
 extern Typetab    *SwigType_pop_scope();
 extern Typetab    *SwigType_set_scope(Typetab *h);
@@ -310,14 +325,18 @@ extern SwigType *Swig_symbol_typedef_reduce(SwigType *ty, Symtab *tab);
 /* Parameters are really just hidden behind a DOH object.  The following
    interface will probably be simplified even further. */
 
-extern Parm       *NewParm(SwigType *type, String_or_char *n);
+extern Parm       *NewParm(SwigType *type, const String_or_char *n);
 extern Parm       *CopyParm(Parm *p);
 extern ParmList   *CopyParmList(ParmList *);
 extern int         ParmList_len(ParmList *);
 extern int         ParmList_numarg(ParmList *);
 extern int         ParmList_numrequired(ParmList *);
 extern String     *ParmList_str(ParmList *);
+extern String     *ParmList_str_defaultargs(ParmList *);
 extern String     *ParmList_protostr(ParmList *);
+extern int         ParmList_is_compactdefargs(ParmList *p);
+extern int         ParmList_has_defaultargs(ParmList *p);
+extern ParmList   *ParmList_copy_all_except_last_parm(ParmList *p);
 
 /* --- Parse tree support --- */
 
@@ -355,6 +374,8 @@ extern void Swig_print_tags(File *obj, Node *root);
 extern void Swig_print_tree(Node *obj);
 extern void Swig_print_node(Node *obj);
 
+extern void Swig_print_xml(Node *obj, String* filename);
+
 /* -- Wrapper function Object */
 
 typedef struct {
@@ -396,13 +417,13 @@ extern void      Swig_name_object_set(Hash *namehash, String_or_char *name, Swig
 extern DOH      *Swig_name_object_get(Hash *namehash, String_or_char *prefix, String_or_char *name, SwigType *decl);
 extern void      Swig_name_object_inherit(Hash *namehash, String *base, String *derived);
 extern void      Swig_features_get(Hash *features, String_or_char *prefix, String_or_char *name, SwigType *decl, Node *n);
-extern void      Swig_feature_set(Hash *features, String_or_char *name, SwigType *decl, String_or_char *fname, String *value);
+extern void      Swig_feature_set(Hash *features, const String_or_char *name, SwigType *decl, const String_or_char *featurename, String *value, Hash *featureattribs);
 
 /* --- Misc --- */
 extern char      *Swig_copy_string(const char *c);
 extern void       Swig_banner(File *f);
 extern String    *Swig_string_escape(String *s);
-extern String    *Swig_string_mangle(String *s);
+extern String    *Swig_string_mangle(const String *s);
 extern String    *Swig_scopename_prefix(String *s);
 extern String    *Swig_scopename_last(String *s);
 extern String    *Swig_scopename_first(String *s);
@@ -412,6 +433,8 @@ extern int        Swig_scopename_check(String *s);
 extern void       Swig_init();
 extern void       Swig_warn(const char *filename, int line, const char *msg);
 
+extern int        Swig_value_wrapper_mode(int mode);
+
 
 #define WARNING(msg) Swig_warn(__FILE__,__LINE__,msg)
 
@@ -428,10 +451,7 @@ extern void       Swig_error_msg_format(ErrorMessageFormat format);
 
 /* --- C Wrappers --- */
 extern String    *Swig_cparm_name(Parm *p, int i);
-extern String    *Swig_clocal(SwigType *t, String_or_char *name, String_or_char *value);
 extern String    *Swig_wrapped_var_type(SwigType *t);
-extern String    *Swig_wrapped_var_deref(SwigType *t, String_or_char *name);
-extern String    *Swig_wrapped_var_assign(SwigType *t, String_or_char *name);
 extern int        Swig_cargs(Wrapper *w, ParmList *l);
 extern String    *Swig_cresult(SwigType *t, const String_or_char *name, const String_or_char *decl);
 
@@ -439,10 +459,16 @@ extern String    *Swig_cfunction_call(String_or_char *name, ParmList *parms);
 extern String    *Swig_cmethod_call(String_or_char *name, ParmList *parms, String_or_char *self);
 extern String    *Swig_cconstructor_call(String_or_char *name);
 extern String    *Swig_cppconstructor_call(String_or_char *name, ParmList *parms);
-extern String    *Swig_cdestructor_call();
-extern String    *Swig_cppdestructor_call();
+extern String    *Swig_unref_call(Node *n);
+extern String    *Swig_ref_call(Node *n, const String* lname);
+extern String    *Swig_cdestructor_call(Node *n);
+extern String    *Swig_cppdestructor_call(Node *n);
 extern String    *Swig_cmemberset_call(String_or_char *name, SwigType *type, String_or_char *self);
-extern String    *Swig_cmemberget_call(String_or_char *name, SwigType *t, String_or_char *self);
+extern String    *Swig_cmemberget_call(const String_or_char *name, SwigType *t, String_or_char *self);
+
+extern String    *Swig_extension_code(const String *function_name,
+ParmList *parms, SwigType* return_type, const String *code, int cplusplus);
+extern int        Swig_add_extension_code(Node *n, const String *function_name, ParmList *parms, SwigType* return_type, const String *code, int cplusplus);
 
 /* --- Transformations --- */
 
@@ -459,6 +485,7 @@ extern int        Swig_VarsetToFunction(Node *n);
 
 #define  CWRAP_EXTEND             0x01
 #define  CWRAP_SMART_POINTER      0x02
+#define  CWRAP_VAR_REFERENCE      0x04
 
 /* --- Director Helpers --- */
 extern Node      *Swig_methodclass(Node *n);
@@ -477,7 +504,7 @@ extern     int    Swig_typemap_apply(ParmList *srcpat, ParmList *destpat);
 extern     void   Swig_typemap_clear_apply(ParmList *pattern);
 extern     void   Swig_typemap_debug();
 
-extern Hash  *Swig_typemap_search(const String_or_char *op, SwigType *type, String_or_char *pname, SwigType **matchtype);
+extern Hash  *Swig_typemap_search(const String_or_char *op, SwigType *type, const String_or_char *pname, SwigType **matchtype);
 extern Hash  *Swig_typemap_search_multi(const String_or_char *op, ParmList *parms, int *nmatch);
 extern String *Swig_typemap_lookup(const String_or_char *op, SwigType *type, String_or_char *pname, String_or_char *lname,
 				  String_or_char *source, String_or_char *target, Wrapper *f);
@@ -491,11 +518,13 @@ extern void   Swig_typemap_attach_parms(const String_or_char *op, ParmList *parm
 
 /* --- Code fragment support --- */
 
-extern  void  Swig_fragment_register(String *name, String *section, String *code);
+extern  void  Swig_fragment_register(Node* fragment);
 extern  void  Swig_fragment_emit(String *name);
 
 /* hacks defined in C++ ! */
 extern  int   Swig_need_protected();
+extern  int   Swig_director_mode();
+extern  int   Swig_template_extmode();
 
 #ifdef __cplusplus
 }
