@@ -12,9 +12,10 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_typesys_c[] = "$Header: /cvsroot/SWIG/Source/Swig/typesys.c,v 1.10 2003/11/26 20:56:44 beazley Exp $";
+char cvsroot_typesys_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/typesys.c,v 1.23 2004/12/04 08:33:39 marcelomatus Exp $";
 
 #include "swig.h"
+#include "cparse.h"
 
 /* -----------------------------------------------------------------------------
  * Synopsis
@@ -104,10 +105,40 @@ static Typetab *global_scope  = 0;                /* The global scope
 static Hash    *scopes        = 0;                /* Hash table containing fully qualified scopes */
 
 /* Performance optimization */
+#define SWIG_TYPEDEF_RESOLVE_CACHE 
 static Hash     *typedef_resolve_cache = 0;
 static Hash     *typedef_all_cache = 0;
 static Hash     *typedef_qualified_cache = 0;
 
+/* common attribute keys, to avoid calling find_key all the times */
+static String *k_name = 0;
+static String *k_qname = 0;
+static String *k_symtab = 0;
+static String *k_using = 0;
+static String *k_scope = 0;
+static String *k_typetab = 0;
+static String *k_inherit = 0;
+static String *k_parent = 0;
+static String *k_value = 0;
+
+/* 
+   Enable this one if your language fully support SwigValueWrapper<T>.
+   
+   Leaving at '0' keeps the old swig behavior, which is not
+   always safe, but is well known.
+
+   Setting at '1' activates the new scheme, which is always safe but
+   it requires all the typemaps ready for that.
+  
+*/
+static int value_wrapper_mode = 0;
+int Swig_value_wrapper_mode(int mode) 
+{
+  value_wrapper_mode = mode;
+  return mode;
+}
+
+
 static void flush_cache() {
   typedef_resolve_cache = 0;
   typedef_all_cache = 0;
@@ -117,21 +148,32 @@ static void flush_cache() {
 /* Initialize the scoping system */
 
 void SwigType_typesystem_init() {
+  k_name = NewString("name");
+  k_qname = NewString("qname");
+  k_symtab = NewString("symtab");
+  k_using = NewString("using");
+  k_scope = NewString("scope");
+  k_typetab = NewString("typetab");
+  k_inherit = NewString("inherit");
+  k_parent = NewString("parent");
+  k_value = NewString("value");
+
   if (global_scope) Delete(global_scope);
   if (scopes)       Delete(scopes);
 
   current_scope = NewHash();
   global_scope  = current_scope;
 
-  Setattr(current_scope,"name","");              /* No name for global scope */
+  Setattr(current_scope,k_name,"");              /* No name for global scope */
   current_typetab = NewHash();
-  Setattr(current_scope,"typetab", current_typetab);
+  Setattr(current_scope,k_typetab, current_typetab);
 
   current_symtab = 0;
   scopes = NewHash();
   Setattr(scopes,"",current_scope);
 }
 
+
 /* -----------------------------------------------------------------------------
  * SwigType_typedef()
  *
@@ -189,15 +231,15 @@ int SwigType_typedef_class(String_or_char *name) {
 
 String *
 SwigType_scope_name(Typetab *ttab) {
-  String *qname = NewString(Getattr(ttab,"name"));
-  ttab = Getattr(ttab,"parent");
+  String *qname = NewString(Getattr(ttab,k_name));
+  ttab = Getattr(ttab,k_parent);
   while (ttab) {
-    String *pname = Getattr(ttab,"name");
+    String *pname = Getattr(ttab,k_name);
     if (Len(pname)) {
       Insert(qname,0,"::");
       Insert(qname,0,pname);
     }
-    ttab = Getattr(ttab,"parent");
+    ttab = Getattr(ttab,k_parent);
   }
   return qname;
 }
@@ -207,7 +249,7 @@ SwigType_scope_name(Typetab *ttab) {
  * Creates a new scope
  * ----------------------------------------------------------------------------- */
 
-void SwigType_new_scope(String_or_char *name) {
+void SwigType_new_scope(const String_or_char *name) {
   Typetab *s;
   Hash    *ttab;
   String  *qname;
@@ -216,15 +258,15 @@ void SwigType_new_scope(String_or_char *name) {
     name = "<unnamed>";
   }
   s = NewHash();
-  Setattr(s,"name", name);
-  Setattr(s,"parent", current_scope);
+  Setattr(s,k_name, name);
+  Setattr(s,k_parent, current_scope);
   ttab = NewHash();
-  Setattr(s,"typetab", ttab);
+  Setattr(s,k_typetab, ttab);
 
   /* Build fully qualified name and */
   qname = SwigType_scope_name(s);
   Setattr(scopes,qname,s);
-  Setattr(s,"qname",qname);
+  Setattr(s,k_qname,qname);
   Delete(qname);
 
   current_scope = s;
@@ -244,10 +286,10 @@ void
 SwigType_inherit_scope(Typetab *scope) {
   List *inherits;
   int   i, len;
-  inherits = Getattr(current_scope,"inherit");
+  inherits = Getattr(current_scope,k_inherit);
   if (!inherits) {
     inherits = NewList();
-    Setattr(current_scope,"inherit", inherits);
+    Setattr(current_scope,k_inherit, inherits);
   }
   assert(scope != current_scope);
   
@@ -290,10 +332,10 @@ SwigType_using_scope(Typetab *scope) {
   {
     List *ulist;
     int   i, len;
-    ulist = Getattr(current_scope,"using");
+    ulist = Getattr(current_scope,k_using);
     if (!ulist) {
       ulist = NewList();
-      Setattr(current_scope,"using", ulist);
+      Setattr(current_scope,k_using, ulist);
     }
     assert(scope != current_scope);
     len = Len(ulist);
@@ -315,7 +357,7 @@ SwigType_using_scope(Typetab *scope) {
 
 Typetab *SwigType_pop_scope() {
   Typetab *s, *s1;
-  s = Getattr(current_scope,"parent");
+  s = Getattr(current_scope,k_parent);
   if (!s) {
     current_scope = 0;
     current_typetab = 0;
@@ -324,8 +366,8 @@ Typetab *SwigType_pop_scope() {
   }
   s1 = current_scope;
   current_scope = s;
-  current_typetab = Getattr(s,"typetab");
-  current_symtab = Getattr(s,"symtab");
+  current_typetab = Getattr(s,k_typetab);
+  current_symtab = Getattr(s,k_symtab);
   flush_cache();
   return s1;
 }
@@ -341,8 +383,8 @@ SwigType_set_scope(Typetab *t) {
   Typetab *old = current_scope;
   if (!t) t = global_scope;
   current_scope = t;
-  current_typetab = Getattr(t,"typetab");
-  current_symtab = Getattr(t,"symtab");
+  current_typetab = Getattr(t,k_typetab);
+  current_symtab = Getattr(t,k_symtab);
   flush_cache();
   return old;
 }
@@ -355,7 +397,7 @@ SwigType_set_scope(Typetab *t) {
 
 void
 SwigType_attach_symtab(Symtab *sym) {
-  Setattr(current_scope,"symtab",sym);
+  Setattr(current_scope,k_symtab,sym);
   current_symtab = sym;
 }
 
@@ -371,15 +413,15 @@ void SwigType_print_scope(Typetab *t) {
 
   for (i = First(scopes); i.key; i = Next(i)) {
     t = i.item;
-    ttab = Getattr(i.item,"typetab");
+    ttab = Getattr(i.item,k_typetab);
     
     Printf(stdout,"Type scope '%s' (%x)\n", i.key, i.item);
     {
-      List *inherit = Getattr(i.item,"inherit");
+      List *inherit = Getattr(i.item,k_inherit);
       if (inherit) {
 	Iterator j;
 	for (j = First(inherit); j.item; j = Next(j)) {
-	  Printf(stdout,"    Inherits from '%s' (%x)\n", Getattr(j.item,"qname"), j.item);
+	  Printf(stdout,"    Inherits from '%s' (%x)\n", Getattr(j.item,k_qname), j.item);
 	}
       }
     }
@@ -406,7 +448,7 @@ SwigType_find_scope(Typetab *s, String *nameprefix) {
   ss = s;
   while (ss) {
     String *full;
-    String *qname = Getattr(ss,"qname");
+    String *qname = Getattr(ss,k_qname);
     if (qname) {
       full = NewStringf("%s::%s", qname, nameprefix);
     } else {
@@ -425,7 +467,7 @@ SwigType_find_scope(Typetab *s, String *nameprefix) {
     if (!s) {
       /* Check inheritance */
       List *inherit;
-      inherit = Getattr(ss,"using");
+      inherit = Getattr(ss,k_using);
       if (inherit) {
 	Typetab *ttab;
 	int i, len;
@@ -444,35 +486,38 @@ SwigType_find_scope(Typetab *s, String *nameprefix) {
       }
     }
     if (!check_parent) break;
-    ss = Getattr(ss,"parent");
+    ss = Getattr(ss,k_parent);
   }
   if (nnameprefix) Delete(nnameprefix);
   return 0;
 }
 
 /* ----------------------------------------------------------------------------- 
- * SwigType_typedef_resolve()
+ * typedef_resolve()
  *
  * Resolves a typedef and returns a new type string.  Returns 0 if there is no
  * typedef mapping.  base is a name without qualification.
+ * Internal function.
  * ----------------------------------------------------------------------------- */
 
 static Typetab  *resolved_scope = 0;
 
 /* Internal function */
+
 static SwigType *
 typedef_resolve(Typetab *s, String *base) {
   Hash     *ttab;
   SwigType *type;
   List     *inherit;
+  Typetab  *parent;
 
-  if (!s) return 0;
+  /* if (!s) return 0; *//* now is checked bellow */
   /* Printf(stdout,"Typetab %s : %s\n", Getattr(s,"name"), base);  */
 
   if (Getmark(s)) return 0;
   Setmark(s,1);
 
-  ttab = Getattr(s,"typetab");
+  ttab = Getattr(s,k_typetab);
   type = Getattr(ttab,base);
   if (type) {
     resolved_scope = s;
@@ -480,7 +525,7 @@ typedef_resolve(Typetab *s, String *base) {
     return type;
   }
   /* Hmmm. Not found in my scope.  It could be in an inherited scope */
-  inherit = Getattr(s,"inherit");
+  inherit = Getattr(s,k_inherit);
   if (inherit) {
     int i,len;
     len = Len(inherit);
@@ -492,11 +537,17 @@ typedef_resolve(Typetab *s, String *base) {
       }
     }
   }
-  type = typedef_resolve(Getattr(s,"parent"), base);
+  parent = Getattr(s,k_parent);
+  type = parent ? typedef_resolve(parent, base) : 0;
   Setmark(s,0);
   return type;
 }
 
+/* ----------------------------------------------------------------------------- 
+ * SwigType_typedef_resolve()
+ * ----------------------------------------------------------------------------- */
+
+/* #define SWIG_DEBUG */
 SwigType *SwigType_typedef_resolve(SwigType *t) {
   String *base;
   String *type = 0;
@@ -515,31 +566,29 @@ SwigType *SwigType_typedef_resolve(SwigType *t) {
 
   resolved_scope = 0;
 
-  /*
+#ifdef SWIG_TYPEDEF_RESOLVE_CACHE
   if (!typedef_resolve_cache) {
     typedef_resolve_cache = NewHash();
   }
   r = Getattr(typedef_resolve_cache,t);
   if (r) {
-    if (r != noscope) {
-      resolved_scope = Getmeta(r,"scope");
-      return Copy(r);
-    } else {
-      return 0;
-    }
+    resolved_scope = Getmeta(r,k_scope);
+    return Copy(r);
   }
-  */
+#endif
 
   base = SwigType_base(t);
 
-  /* Printf(stdout,"base = '%s'\n", base); */
+#ifdef SWIG_DEBUG
+  Printf(stdout,"base = '%s' t='%s'\n", base, t); 
+#endif
 
   if (SwigType_issimple(base)) {
     s = current_scope;
     ttab = current_typetab;
     if (Strncmp(base,"::",2) == 0) {
       s = global_scope;
-      ttab = Getattr(s,"typetab");
+      ttab = Getattr(s,k_typetab);
       Delitem(base,0);
       Delitem(base,0);
     }
@@ -553,7 +602,9 @@ SwigType *SwigType_typedef_resolve(SwigType *t) {
       if (Swig_scopename_check(base)) {
 	/* A qualified name. */
 	nameprefix = Swig_scopename_prefix(base);
-	/*	Printf(stdout,"nameprefix = '%s'\n", nameprefix);*/
+#ifdef SWIG_DEBUG
+	Printf(stdout,"nameprefix = '%s'\n", nameprefix); 
+#endif
 	if (nameprefix) {
 	  /* Name had a prefix on it.   See if we can locate the proper scope for it */
 	  s = SwigType_find_scope(s,nameprefix);
@@ -567,17 +618,32 @@ SwigType *SwigType_typedef_resolve(SwigType *t) {
 	  }
 	  /* Try to locate the name starting in the scope */
 	  namebase = Swig_scopename_last(base);
-	  /* Printf(stdout,"namebase = '%s'\n", namebase); */
+#ifdef SWIG_DEBUG
+	  Printf(stdout,"namebase = '%s'\n", namebase); 
+#endif
 	  type = typedef_resolve(s,namebase);
-	  /* Printf(stdout,"%s type = '%s'\n", Getattr(s,"name"), type); */
-	  if ((type) && (!Swig_scopename_check(type))) {
+	  if (type) {
+	    /* we need to look for the resolved type, this will also
+	       fix the resolved_scope if 'type' and 'namebase' are
+	       declared in different scopes */
+	    String *rtype = 0;
+	    rtype = typedef_resolve(resolved_scope,type);
+	    if (rtype) type = rtype;
+	  }
+#ifdef SWIG_DEBUG
+	  Printf(stdout,"%s type = '%s'\n", Getattr(s,"name"), type); 
+#endif
+	  if ((type) && (!Swig_scopename_check(type)) && resolved_scope) {
 	    Typetab *rtab = resolved_scope;
-	    String *qname = Getattr(resolved_scope,"qname");
+	    String *qname = Getattr(resolved_scope,k_qname);
 	    /* If qualified *and* the typename is defined from the resolved scope, we qualify */
 	    if ((qname) && typedef_resolve(resolved_scope,type)) {
 	      type = Copy(type);
 	      Insert(type,0,"::");
 	      Insert(type,0,qname);
+#ifdef SWIG_DEBUG
+	      Printf(stdout,"qual %s \n", type);
+#endif
 	      newtype = 1;
 	    } 
 	    resolved_scope = rtab;
@@ -717,22 +783,20 @@ SwigType *SwigType_typedef_resolve(SwigType *t) {
   }
 
  return_result:
-  /*
+#ifdef SWIG_TYPEDEF_RESOLVE_CACHE
   {
     String *key = NewString(t);
-    if (!r) {
-      Setattr(typedef_resolve_cache,key,noscope);
-    } else {
+    if (r) {
       SwigType *r1;
       Setattr(typedef_resolve_cache,key,r);
-      Setmeta(r,"scope",resolved_scope);
+      Setmeta(r,k_scope,resolved_scope);
       r1 = Copy(r);
       Delete(r);
       r = r1;
     }
     Delete(key);
   }
-  */
+#endif
   return r;
 }
 
@@ -746,6 +810,7 @@ SwigType *SwigType_typedef_resolve_all(SwigType *t) {
   SwigType *n;
   SwigType *r;
 
+  /* Check to see if the typedef resolve has been done before by checking the cache */
   if (!typedef_all_cache) {
     typedef_all_cache = NewHash();
   }
@@ -753,11 +818,15 @@ SwigType *SwigType_typedef_resolve_all(SwigType *t) {
   if (r) {
     return Copy(r);
   }
+
+  /* Recursively resolve the typedef */
   r = NewString(t);
   while ((n = SwigType_typedef_resolve(r))) {
     Delete(r);
     r = n;
   }
+
+  /* Add the typedef to the cache for next time it is looked up */
   {
     String *key;
     SwigType *rr = Copy(r);
@@ -782,7 +851,6 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
   List   *elements;
   String *result;
   int     i,len;
-
   if (!typedef_qualified_cache) typedef_qualified_cache = NewHash();
   result = Getattr(typedef_qualified_cache,t);
   if (result) {
@@ -799,21 +867,16 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
       if (!SwigType_istemplate(e)) {
 	String *isenum = 0;
 	if (SwigType_isenum(e)) {
-	  isenum = e;
+	  isenum = NewString("enum ");
 	  e = NewString(Char(e)+5);
 	}
 	resolved_scope = 0;
 	if (typedef_resolve(current_scope,e)) {
 	  /* resolved_scope contains the scope that actually resolved the symbol */
-	  String *qname = Getattr(resolved_scope,"qname");
+	  String *qname = Getattr(resolved_scope,k_qname);
 	  if (qname) {
 	    Insert(e,0,"::");
 	    Insert(e,0,qname);
-	    if (isenum) {
-	      Clear(isenum);
-	      Printf(isenum, "enum %s", e);
-	      Delete(e);
-	    }
 	  }
 	} else {
 	  if (Swig_scopename_check(e)) {
@@ -834,7 +897,7 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
 	    /* It's a bare name.  It's entirely possible, that the
                name is part of a namespace. We'll check this by unrolling
                out of the current scope */
-	    
+	     
 	    Typetab *cs = current_scope;
 	    while (cs) {
 	      String *qs = SwigType_scope_name(cs);
@@ -849,11 +912,14 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
 		break;
 	      }
 	      Delete(qs);
-	      cs = Getattr(cs,"parent");
+	      cs = Getattr(cs,k_parent);
 	    }
 	  }
 	}
-	if (isenum) e = isenum;
+
+	if (isenum) {
+          Insert(e,0,isenum);
+        }
 
       } else {
 	/* Template.  We need to qualify template parameters as well as the template itself */
@@ -869,7 +935,7 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
 	pi = First(parms);
 	while ((p = pi.item)) {
 	  String *qt = SwigType_typedef_qualified(p);
-	  if ((Strcmp(qt,p) == 0)) { /*  && (!Swig_scopename_check(qt))) { */
+	  if ((Strcmp(qt,p) == 0)) { /*  && (!Swig_scopename_check(qt))) */
 	    /* No change in value.  It is entirely possible that the parameter is an integer value.
 	       If there is a symbol table associated with this scope, we're going to check for this */
 
@@ -886,7 +952,7 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
 		    String *qn = Swig_symbol_qualified(n);
 		    if (Len(qn)) {
 		      Append(qn,"::");
-		      Append(qn,Getattr(n,"name"));
+		      Append(qn,Getattr(n,k_name));
 		      Delete(value);
 		      value = qn;
 		      continue;
@@ -894,9 +960,9 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
 		      Delete(qn);
 		      break;
 		    }
-		  } else if ((Strcmp(nodeType(n),"cdecl") == 0) && (Getattr(n,"value"))) {
+		  } else if ((Strcmp(nodeType(n),"cdecl") == 0) && (Getattr(n,k_value))) {
 		    Delete(value);
-		    value = Copy(Getattr(n,"value"));
+		    value = Copy(Getattr(n,k_value));
 		    continue;
 		  }
 		}
@@ -1028,24 +1094,24 @@ int SwigType_typedef_using(String_or_char *name) {
   td = SwigType_typedef_resolve(name);
   /*  Printf(stdout,"td = '%s' %x\n", td, resolved_scope); */
   if (resolved_scope) {
-    defined_name = Getattr(resolved_scope,"qname");
+    defined_name = Getattr(resolved_scope,k_qname);
     if (defined_name) {
       defined_name = Copy(defined_name);
       Append(defined_name,"::");
       Append(defined_name,base);
+      /*  Printf(stdout,"defined_name = '%s'\n", defined_name);*/
+      tt = SwigType_find_scope(current_scope,defined_name);
     }
   }
   if (td) Delete(td);
 
-  /*  Printf(stdout,"defined_name = '%s'\n", defined_name);*/
-  tt = SwigType_find_scope(current_scope,defined_name);
 
   /* Figure out the scope the using directive refers to */
   {
     prefix = Swig_scopename_prefix(name);
     s = SwigType_find_scope(current_scope,prefix);
     if (s) {
-      Hash *ttab = Getattr(s,"typetab");
+      Hash *ttab = Getattr(s,k_typetab);
       if (!Getattr(ttab,base) && defined_name) {
 	Setattr(ttab,base, defined_name);
       }
@@ -1143,6 +1209,10 @@ int SwigType_type(SwigType *t)
   if (strcmp(c,"unsigned char") == 0) return T_UCHAR;
   if (strcmp(c,"float") == 0) return T_FLOAT;
   if (strcmp(c,"double") == 0) return T_DOUBLE;
+  if (strcmp(c,"long double") == 0) return T_LONGDOUBLE;
+  if (!cparse_cplusplus && (strcmp(c,"float complex") == 0)) return T_FLTCPLX;
+  if (!cparse_cplusplus && (strcmp(c,"double complex") == 0)) return T_DBLCPLX;
+  if (!cparse_cplusplus && (strcmp(c,"complex") == 0)) return T_COMPLEX;
   if (strcmp(c,"void") == 0) return T_VOID;
   if (strcmp(c,"bool") == 0) return T_BOOL;
   if (strcmp(c,"long long") == 0) return T_LONGLONG;
@@ -1161,6 +1231,91 @@ int SwigType_type(SwigType *t)
   return T_USER;
 }
 
+/* -----------------------------------------------------------------------------
+ * SwigType_alttype()
+ *
+ * Returns the alternative value type needed in C++ for class value
+ * types. When swig is not sure about using a plain $ltype value,
+ * since the class doesn't have a default constructor, or it can't be
+ * assigned, you will get back 'SwigValueWrapper<type >'.
+ *
+ * This is the default behavior unless:
+ *
+ *  1.- swig detects a default_constructor and 'setallocate:default_constructor'
+ *      attribute.
+ *
+ *  2.- swig doesn't mark 'type' as noassignable.
+ *
+ *  3.- the user specify that the value wrapper is not needed by using
+ *      the %feature("novaluewrapper"), in that case the user need to type
+ *
+ *        %feature("novaluewrapper") MyOpaqueClass;
+ *        class MyOpaqueClass;
+ *
+ * Users can also force the use of the value wrapper by using the
+ * %feature("valuewrapper"). 
+ * ----------------------------------------------------------------------------- */
+
+SwigType *SwigType_alttype(SwigType *t, int local_tmap) {
+  if (!cparse_cplusplus) return 0;
+  if (value_wrapper_mode == 0) {
+    /* old partial use of SwigValueTypes, it can fail for opaque types */
+    Node *n;
+    SwigType *w = 0;
+    int use_wrapper = 0;
+    if (local_tmap) return 0;
+    if (!use_wrapper && SwigType_isclass(t)) {
+      SwigType *ftd = SwigType_typedef_resolve_all(t);
+      SwigType *td = SwigType_strip_qualifiers(ftd);
+      Delete(ftd);
+      if ((n = Swig_symbol_clookup(td,0))) {
+	if (Getattr(n,"feature:valuewrapper")) {
+	  use_wrapper = 1;
+	} else {
+	  if ((Strcmp(nodeType(n),"class") == 0) 
+	      && (!Getattr(n,"allocate:default_constructor") 
+		  || (Getattr(n,"allocate:noassign")))) {
+	    use_wrapper = !Getattr(n,"feature:novaluewrapper") 
+	      || Getattr(n,"feature:nodefault");
+	  }
+	}
+      } else {
+	if (SwigType_issimple(td) && SwigType_istemplate(td)) {
+	  use_wrapper = 1;
+	}
+      }
+      if (use_wrapper) w = NewStringf("SwigValueWrapper<%s >",SwigType_str(td,0));
+      Delete(td);
+    }
+    return w;
+  } else {
+    /*  safe use of SwigValueTypes, it can fail with some typemaps */
+    SwigType *w = 0;
+    Node *n = 0;
+    int use_wrapper = 1;
+    SwigType *ftd = SwigType_typedef_resolve_all(t);
+    SwigType *td = SwigType_strip_qualifiers(ftd);
+    if (SwigType_type(td) == T_USER) {
+      if ((n = Swig_symbol_clookup(td,0))) {
+	if (((Strcmp(nodeType(n),"class") == 0) 
+	     && !Getattr(n,"allocate:noassign")
+	     && (Getattr(n,"allocate:default_constructor")))
+	    || (Getattr(n,"feature:novaluewrapper"))) {
+	  use_wrapper = Getattr(n,"feature:valuewrapper") ? 1 : 0;
+	}
+      }
+    } else {
+      use_wrapper = 0;
+    }    
+    if (use_wrapper) {
+      w = NewStringf("SwigValueWrapper<%s >",td);
+    }
+    Delete(ftd);
+    Delete(td);
+    return w;    
+  }
+}
+
 /******************************************************************************
  ***                         * * * WARNING * * *                            ***
  ***                                                                        ***
@@ -1392,44 +1547,20 @@ List *SwigType_equivalent_mangle(String *ms, Hash *checked, Hash *found) {
  * ----------------------------------------------------------------------------- */
 
 static
-String *SwigType_clientdata_collect(String *ms, Hash *checked) {
-  Hash *ch;
+String *SwigType_clientdata_collect(String *ms) {
   Hash *mh;
   String *clientdata = 0;
 
-  if (checked) {
-    ch = checked;
-  } else {
-    ch = NewHash();
-  }
-  if (Getattr(ch,ms)) goto check_exit;    /* Already checked this type */
-  Setattr(ch, ms, "1");
   mh = Getattr(r_mangled,ms);
   if (mh) {
     Iterator ki;
     ki = First(mh);
     while (ki.key) {
-      Hash *rh;
-      Setattr(ch,ki.key,"1");
       clientdata = Getattr(r_clientdata,ki.key);
-      if (clientdata) goto check_exit;
-      rh = Getattr(r_resolved,ki.key);
-      if (rh) {
-	Iterator rk;
-	rk = First(rh);
-	while (rk.key) {
-	  clientdata = SwigType_clientdata_collect(rk.key,ch);
-	  if (clientdata) goto check_exit;
-	  rk = Next(rk);
-	}
-      }
+      if (clientdata) break;
       ki = Next(ki);
     }
   }
- check_exit:
-  if (!checked) {
-    Delete(ch);
-  }
   return clientdata;
 }
 
@@ -1643,6 +1774,9 @@ SwigType_emit_type_table(File *f_forward, File *f_table) {
   Printf(stdout,"---conversions---\n");
   Printf(stdout,"%s\n", conversions);
 
+  Printf(stdout,"---r_clientdata---\n");
+  Printf(stdout,"%s\n", r_clientdata);
+
 #endif
   table = NewString("");
   types = NewString("");
@@ -1653,14 +1787,29 @@ SwigType_emit_type_table(File *f_forward, File *f_table) {
   while (ki.key) {
     List *el;
     Iterator ei;
-    String *cd;
+    SwigType *lt;
+    SwigType *rt;
+    String *nt;
+    String *ln;
+    String *rn;
+    const String *cd;
 
     Printf(f_forward,"#define  SWIGTYPE%s swig_types[%d] \n", ki.key, i);
     Printv(types,"static swig_type_info _swigt_", ki.key, "[] = {", NIL);
     
-    cd = SwigType_clientdata_collect(ki.key,0);
+    cd = SwigType_clientdata_collect(ki.key);
     if (!cd) cd = "0";
-    Printv(types,"{\"", ki.key, "\", 0, \"", SwigType_str(Getattr(r_ltype,ki.key),0),"\", ", cd, "},", NIL);
+    lt = Getattr(r_ltype,ki.key);
+    rt = SwigType_typedef_resolve_all(lt);
+    /* we save the original type and the fully resolved version */
+    ln = SwigType_lstr(lt,0);
+    rn = SwigType_lstr(rt,0);
+    if (Strcmp(ln,rn) == 0) {
+      nt = NewStringf("%s", ln);
+    } else {
+      nt = NewStringf("%s|%s", rn, ln);
+    }
+    Printv(types,"{\"", ki.key, "\", 0, \"",nt,"\", ", cd, ", 0, 0, 0},", NIL);
     el = SwigType_equivalent_mangle(ki.key,0,0);
     for (ei = First(el); ei.item; ei = Next(ei)) {
       String *ckey;
@@ -1668,14 +1817,16 @@ SwigType_emit_type_table(File *f_forward, File *f_table) {
       ckey = NewStringf("%s+%s", ei.item, ki.key);
       conv = Getattr(conversions,ckey);
       if (conv) {
-	Printf(types,"{\"%s\", %s},", ei.item, conv);
+	Printf(types,"{\"%s\", %s, 0, 0, 0, 0, 0},", ei.item, conv);
       } else {
-	Printf(types,"{\"%s\"},", ei.item);
+	Printf(types,"{\"%s\", 0, 0, 0, 0, 0, 0},", ei.item);
       }
       Delete(ckey);
     }
     Delete(el);
-    Printf(types,"{0}};\n");
+    Delete(nt);
+    Delete(rt);
+    Printf(types,"{0, 0, 0, 0, 0, 0, 0}};\n");
     Printv(table, "_swigt_", ki.key, ", \n", NIL);
     ki = Next(ki);
     i++;
